#include <stdio.h>

#include "binding.h"
#include "errors.h"
#include "globals.h"

int getBindValue(Bind_mat bind, int row, int col);	/*prototype */
void setBindValue(Bind_mat bind, int row, int col, int val);

#define   matIndex(column_dim, row, col) (col-1) * (column_dim) + (row-1)

Bind_mat createBindingMatrix(int dim)
{
	Bind_mat bind;
	
	bind = emalloc(sizeof(Bind_str), "Binding Structure");
	bind->b_vect = (int *) ecalloc(dim*dim, sizeof(int), "Binding Mat.");
	bind->dim_b = dim;

	return bind;
}

Bind_mat sCreateBindingMatrix(int * bind_arr, int dim)
{
	Bind_mat bind;
	
	bind = emalloc(sizeof(Bind_str), "Binding Structure");
	bind->dim_b = dim;
	bind->b_vect = bind_arr;

	return bind;
}


void computeBinding(I_vect suppress, Cl_vect clust_vect, 
		    Bind_mat bind)
{
	int row, col;
	int temp_r, temp_c;	/* for traversal */
	int new_bind;
	int dim = bind->dim_b;

	for(row = 1; row < dim; row++)
	{
	    if (isSuppressed(suppress,row))
		continue;			/* do nothing */
	    if (isClustered(clust_vect, row) &&
	    	 !isClustHead(clust_vect, row))
		continue;			/* again, nothing to do */
	   
	    /* 
	    now two situations remain:
	    - either it is clustered (and also the cluster head)
	    - or it is not clustered at all 
	    In both the cases, it now depends upon the col being the
	    leader (or head) of its cluster
	    The col might, or might not, be in a cluster
	    */

	    for (col = row+1; col <= dim; col++)
	    {
		if (isSuppressed(suppress, col))
		    continue;			/* do nothing */
		
		if (!isClustered(clust_vect, row) && 
		    !isClustered(clust_vect,col))	
		   continue;		/* since none is clustered */
		
		/*
		now at least one of them is in a cluster. Conditions:
		1. both are in cluster
		2. row in cluster, col not
		3. col in cluster, row not
		Additionally, if row in cluster => cluster head
		*/

		if (isClustered(clust_vect, row) &&
		    isClustered(clust_vect, col) &&
		    !isClustHead(clust_vect, col))
		   continue;	/* no dealing with unimportant*/

		if (isClustered(clust_vect, row) &&
		    isClustered(clust_vect, col))
		{
		    /* compute ALL interactions */
		    new_bind = 0;

		    for (temp_r = row; temp_r <= dim; temp_r++)
		    {
			if (inSameCluster(clust_vect, row, temp_r))
			{
			   for (temp_c = col; temp_c <= dim; temp_c++)
			   {
				if (inSameCluster(clust_vect, col, temp_c))
				   new_bind += getBindValue(bind, temp_r, temp_c);
			   }
			}
		    }
		    setBindValue(bind, row, col, new_bind);
		    continue;
		    
		}

		/*
		Now one, and ONLY one, of row/col is in cluster
		col may or may not be the head
		The other element is always independent (no cluster)
		*/

		if (isClustered(clust_vect, row))
		{
		   /* which means that the col is an independent element */
	
		   new_bind = 0;
		   for (temp_r = row; temp_r <= dim; temp_r++)
		   {
			if (inSameCluster(clust_vect, row, temp_r))
			   new_bind += getBindValue(bind, temp_r, col);
		   }
		   setBindValue(bind, row, col, new_bind);
		   continue;
		}

		if (isClustered(clust_vect, col))
		{
		   /* that is, row is not clustered at all */
		  
		   if (isClustHead(clust_vect, col))
		   {
			new_bind = 0;
			for (temp_c = col; temp_c <= dim; temp_c++)
			{
			    if (inSameCluster(clust_vect, col, temp_c))
			       new_bind += getBindValue(bind, row, temp_c);
			}
			setBindValue(bind, row, col, new_bind);
			continue;
		   }
		}

		else
		   printf("computeBinding: all cases not considered \n");

	    }	/* col for loop */
	}	/* row for loop */

}



void computeBindingSum(Bind_mat bind, 
			I_vect suppress,
		        I_vect bind_sum)
{
	int row, col;
	int sum;
	int dim = bind->dim_b;

	resetIntVector(bind_sum, ABSURD);

	for (row = 1; row <= dim; row++)
	{
	    if (isSuppressed(suppress, row))
		continue;

	    sum = 0;

	    for(col = 1; col <= dim; col++)	/* getBindValue takes care */
	    {
		if (isSuppressed(suppress, col))
		   continue;
	        
		sum += getBindValue(bind, row, col);
	    }

	    /* assign this sum to bind_sum */

	    setIntVector(bind_sum, row, sum);
	}
}


int getBindingSum(I_vect bind_sum, int index)
{
	return getIntValue(bind_sum, index);
}

int getBindValue(Bind_mat bind, int row, int col)
{
	int temp;
	int dim = bind->dim_b;

	if (row == col)
	   return ABSURD;
	if (col < row)		/* only upper diagonal is active */
	{
	   temp = col;
	   col = row;
	   row = temp;
	}	

	return bind->b_vect[matIndex(dim, row, col)];
}


void setBindValue(Bind_mat bind, int row, int col, int val)
{
	int temp;
	int dim = bind->dim_b;

	if (row == col)
	   myError("Trying to manipulate diagonal in Binding\n");
	
	if (col < row)
	{
	   temp = col;
	   col = row;
	   row = temp;
	}

        bind->b_vect[matIndex(dim, row, col)] = val;
}


void printBindMatrix(Bind_mat bind, I_vect suppress)
{
	int row, col;
	int val;
	int dim = bind->dim_b;

	printf("Printing Binding Matrix ....\n");

	for (row = 1; row <= dim; row ++)
	{
	    if (isSuppressed(suppress, row))
		continue;
	    for (col = 1; col <=dim; col++)
	    {
		if (isSuppressed(suppress, col))
		   continue;
		val = getBindValue(bind, row, col);
		printf("%d ", val);
	    }
	    printf("\n");
	}
}



void readIntoBindMatrix(Bind_mat bind, int row, int col, int val)
{
	int dim = bind->dim_b;

        bind->b_vect[matIndex(dim, row, col)] = val;
}
