##############################################################
# Delta functions
##############################################################
# Assumptions
#
#   d1 and d2 are pairwise distance matrices
# For our current situation
# they should be called as follows
#
#  delta.1((1 - fMatch), pDistance)
#
# The following three delta functions are from the paper and are
# originally from Jardine and Sibson.

delta.1 <- function(d1, d2) {
   max(abs(d1 - d2))
}

delta.2 <- function(d1, d2) {
  nelems <- nrow(d1)
  numerator <- sqrt(sum( (d1 - d2) ^ 2 ))
  denominator <- sqrt(0.5 * nelems * (nelems - 1))
  numerator/denominator
}

delta.3 <- function(d1, d2) {
  nelems <- nrow(d1)
  numerator <- sum( abs(d1 - d2) )
  denominator <- 0.5 * nelems * (nelems-1)
  numerator/denominator
}

# The following function is the "hook" into the above delta functions.
delta <- function(fMatch, pDistance)
{
	delta.3(1 - fMatch, normalize(pDistance))
}

##############################################################
# Intersection ratio
##############################################################
# The following function is the intersection ratio defined in the
#	paper.
intersectionRatio <- function(functions.classified, functions.in.system)
{
	functions.classified / functions.in.system
}

##############################################################
# Normalize
##############################################################
# The following function is to normalize a pairwise distance matrix.
#
# Assumptions:
# The matrix contains real numbers that we want to put in the range of
# 	zero to one.  The most similar pair is represented by zero,
#	i.e., this is a dissimilarity matrix.
# This function will make the maximum valued cell one and the minimum
#	valued cell zero.
#
# Note:  This function could be changed for a similarity matrix by
#	subtracting the current return value from one.
#
normalize <- function(value)
{

   # scale values to between 0 and 1
   # The values are assumed to be greater than 0

     s.value <- value / (1 + value)

   # stretch the range of values such that
   # the min is 0 and max is 1

     dmin <- min(s.value)
     spread <- diff(range(s.value))

    if (spread > 0.0001) {
       abs((s.value - dmin) / spread)
    } else {
       # when all elements are very similar
       # return a matrix of 1s (ones)
       # create a matrix of 0s and add 1
       s.value - s.value + 1
    }
}
