
# Functions to maintain the database of experimental data
# Look at the file parameters.S for the parameters to the experiment

# The database is contained in the variable
#      clustsys.data

# It consists of a list of data elements for different types
# of query.  The query names are the same as given on the command line
# of clustsys  .
# For instance : fields, all

# Each data element for a query consists of another list
# of elements. This is the list of variables for which information is
# collected. The variables are:
#      functions.used  : number of functions appearing in the cluster
#      functions.in.system : number of functions in the system
#      goodness   : The goodness number for the clustering
#      system.goodness : The factored goodness

# The data element for each of the variable consists of a matrix
# whose rows are indexed by the key used for that system
# and whose columns are indexed by the method identifier
# For instance:
#    Rows: cs453110, cs453111, .., slicer
#    Columns: e, r, he, hr, hb
#
#  The rows and columns are defined by the experiment parameter
# variables:
#    exp.param$clustering.methods        -- defined in parameter.S
#    exp.param$systems.clustered         --    ""
#    exp.param$querytype                 --    ""

#  Before the experiment is started the variable
#  clustsys.data should have been defined. Else we will get an error
#  It may be defined by 
#      clustsys.data <- list()
#

# The data from a specific observation may be got by
#
#     clustsys.data[[querytype]][[variable]][key,method]
#
# There are two functions available to access and modify a specific
# observation. They are:
#    getVariableValue(sysparam, variable) 
#    putVariableValue(sysparam,  variable, value)
#
#  where sysparam is a list containing fields
#  method, key, and querytype 


init.clustsys.data <- function () {
   clustsys.data <<- exp.param
}

valid.queries <- function (database) {
   v <- c()
   for (aquery in database$querytype) {
      if (!is.null(database[[aquery]])) 
         v <- c(v, aquery)
   }
}

initVariableRecord <- function(systems, methods) {
    nsys <- length(systems)
    nmethods <- length(methods)
    matrix(NA, nrow=nsys, ncol=nmethods, dimnames=list(systems, methods))
}

initialQueryRecord <- function(...) {
    m <- initVariableRecord(...)
    list(functions.used=m, functions.in.system=m, goodness=m, system.goodness=m)
}

existsQueryRecord <- function(querytype) {
    !is.null(clustsys.data[[querytype]])
}

existsKeyEntry <- function(sysparam) {
   any(exp.param$systems.clustered == sysparam$key)
}


createQueryRecord <- function(querytype) {
    clustsys.data[[querytype]] <<- 
              initialQueryRecord(methods= exp.param$clustering.methods,
                                 systems= exp.param$systems.clustered)
    querytype
}

putVariableValue <- function(variablename, value, sysparam) {

   # get system parameters
    querytype <- sysparam$querytype

    key <- sysparam$key
    method <- sysparam$method

   # create a query record if it does not exist
    if (!existsQueryRecord(querytype)) createQueryRecord(querytype)

   # put value

   clustsys.data[[querytype]][[variablename]][key, method] <<- value
   value
}

getVariableValue <- function(variablename, value, sysparam) {

   # get system parameters
    querytype <- sysparam$querytype
    key <- sysparam$key
    method <- sysparam$method

   # create a query record if it does not exist
    if (!existsQueryRecord(querytype)) createQueryRecord(querytype)

   # get value

   clustsys.data[[querytype]][[variablename]][key, method]
}



record.h <- function(value, ...) {
   h <<- value
}

record.functions.in.system <- function(...) {
   putVariableValue("functions.in.system", ...)
}

record.functions.used <- function(...) {
   putVariableValue("functions.used", ...)
}

record.goodness <- function(...) {
   putVariableValue("goodness", ...)
}

record.system.goodness <- function(...) {
   putVariableValue("system.goodness", ...)
}

record.variables <- 
        function(functions.in.system= NA, 
                 functions.used= NA,
	         goodness = NA,
                 system.goodness = NA,
                 sysparam) {

     if (!is.na(functions.in.system)) record.functions.in.system(functions.in.system, sysparam)
     if (!is.na(functions.used)) record.functions.used(functions.used, sysparam)
     if (!is.na(goodness)) record.goodness(goodness, sysparam)
     if (!is.na(system.goodness)) record.system.goodness(system.goodness, sysparam)
}
                 

na.systems <- function(database = clustsys.data) {

    rows.with.na <- c()
    for (querytype in valid.queries(database)) {
      d <- database[[querytype]]
	 for (variable in names(d)) {
	    v <- d[[variable]]
	    rows.with.na <- c(rows.with.na, unlist(na.rows(as.matrix(v))))
	 }
    }

  if (!is.null(rows.with.na))  unique(rows.with.na)
}

wipe.na <- function (database=clustsys.data) {

   # Systems having NA as some value should be dropped
   systems.with.na <- na.systems(database)

  if (!is.null(systems.with.na)) { 
   # Get the systems whose values should be kept
   newdatabase <- drop.systems(database, systems.with.na)
   newdatabase$year <- paste(newdatabase$year, "-NA", sep="")
   newdatabase
  } else database
}

drop.systems <- function(database=clustsys.data,
                         systems.to.drop) {

   newdatabase <- database
   keep.systems <- drop.list.elements(database$systems.clustered, systems.to.drop)
    for (querytype in valid.queries(database)) {
      d <- database[[querytype]]
	 for (variable in names(d)) {
	    v <- d[[variable]]
	    newdatabase[[querytype]][[variable]] <- v[keep.systems,]
	 }
    }
  newdatabase$systems.clustered <- keep.systems
  newdatabase$system.abbr        <- paste(1:length(keep.systems))
  newdatabase$select.programs <- 1:length(keep.systems)
  newdatabase
}


function.ratios <- 
    function(...) {

    d <- extract.a.dataset(...)
    f.used <- d[,"functions.used", drop=F]
    f.in.system <- d[,"functions.in.system", drop=F]
    f.ratios <- f.used/f.in.system
    dimnames(f.ratios)[[2]] <-  "function.ratio"
    f.ratios
    
}


extract.across <- 
      function(variable, database=clustsys.data) 
{
    new.data <- list()

    for (querytype in valid.queries(database)) {
	 new.data[[querytype]] <- 
	  database[[querytype]][[variable]]
    }
  new.data
}

extract.a.variable <-
    function(database=clustsys.data,
             variable="functions.used",
             method="e"
            )
{
     querytypes <- valid.queries(database)
     new.data <- data.frame(database[[querytypes[1]]] [[variable]][,method])

     if (length(querytypes) > 1) 
     for ( i in querytypes[2:length(querytypes)] ) {
        new.data <- data.frame(new.data, database[[i]] [[variable]][,method])
     }

   dimnames(new.data)[[2]] <- querytypes

   new.data
}

extract.a.dataset <-
    function(database=clustsys.data,
             querytype="all",
             method="e"
            )
{

     qdata <- database[[querytype]]

     if (! is.null(qdata)) {
	 variables <- names(qdata)

	 new.data <- data.frame(qdata[[variables[1]]][,method])

	 for ( i in variables[2:length(variables)] ) {
	    new.data <- data.frame(new.data, qdata[[i]][,method])
	 }

       dimnames(new.data)[[2]] <- variables

       new.data
    } else data.frame()
}

transfer.clustsys.data <- function(database) {
 # This function is useful when transferring data from
 # one run of an experiment to another, in the event that
 # the new run has all the systems that the previous run 
 # had, and some more

 # The list "clustsys.data" should exist, before
 # this function is called.

    for (querytype in valid.queries(database)) {
      d <- database[[querytype]]

      if (!existsQueryRecord(querytype)) createQueryRecord(querytype)

      old.dimensions <- dimnames(d[[1]])
      old.systems    <- old.dimensions[[1]]
      old.methods    <- old.dimensions[[2]]

      for (variable in names(d)) {
        clustsys.data[[querytype]][[variable]][old.systems, old.methods] <<-
               d[[variable]]
      }
   }
  list()
}

